package org.artifactory.storage.db.fs.model;

import com.google.common.util.concurrent.MoreExecutors;
import lombok.extern.slf4j.Slf4j;
import org.artifactory.api.context.ContextHelper;
import org.artifactory.storage.fs.VfsException;
import org.artifactory.storage.fs.service.FileService;
import org.artifactory.storage.fs.service.NodeMetaInfoService;
import org.artifactory.storage.fs.service.PropertiesService;
import org.artifactory.storage.fs.service.StatsService;
import org.artifactory.storage.fs.service.WatchesService;
import org.jfrog.common.ExecutionUtils;
import org.springframework.stereotype.Component;

import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;

/**
 * @author Eli Skoran 02/03/2021
 */

@Slf4j
@Component
public class DbMutableItemDeleteHelper {

    public void deleteItem(long id) {
        getWatchesService().deleteWatches(id);
        getPropertiesService().deleteProperties(id);
        getNodeMetaInfoService().deleteMetaInfo(id);
        try {
            ExecutorService currentThreadExecutorService = MoreExecutors.newDirectExecutorService();
            ExecutionUtils.retry(
                    () -> {
                        getStatsService().deleteStats(id);
                        getFileService().deleteItem(id);
                        return null;
                    }, ExecutionUtils.RetryOptions.builder()
                            .numberOfRetries(1)
                            .failedLog("Delete item has failed due to: ${ERROR}. Not trying again")
                            .build(),
                    currentThreadExecutorService).get();
        } catch (InterruptedException | ExecutionException e) {
            log.warn("Delete item has failed due to {}", e.getMessage());
            throw new VfsException(e.getMessage());
        }
    }

    private FileService getFileService() {
        return ContextHelper.get().beanForType(FileService.class);
    }

    private PropertiesService getPropertiesService() {
        return ContextHelper.get().beanForType(PropertiesService.class);
    }

    private WatchesService getWatchesService() {
        return ContextHelper.get().beanForType(WatchesService.class);
    }

    private StatsService getStatsService() {
        return ContextHelper.get().beanForType("statsServiceImpl", StatsService.class);
    }

    private NodeMetaInfoService getNodeMetaInfoService() {
        return ContextHelper.get().beanForType(NodeMetaInfoService.class);
    }
}
