package org.artifactory.storage.db.aql.itest.service.optimizer;

import org.artifactory.aql.model.AqlDomainEnum;
import org.artifactory.aql.model.AqlField;
import org.artifactory.aql.model.AqlPhysicalFieldEnum;
import org.artifactory.aql.model.AqlValue;
import org.artifactory.aql.model.AqlVariableTypeEnum;
import org.artifactory.storage.db.aql.itest.service.AqlAbstractServiceTest;
import org.artifactory.storage.db.aql.sql.builder.query.aql.AqlQuery;
import org.artifactory.storage.db.aql.sql.builder.query.aql.AqlQueryElement;
import org.artifactory.storage.db.aql.sql.builder.query.aql.ComplexPropertyCriterion;
import org.artifactory.storage.db.aql.sql.builder.query.aql.ComplexPropertyCriterionSubSelect;
import org.artifactory.storage.db.aql.sql.builder.query.aql.SimplePropertyCriterion;
import org.artifactory.storage.db.aql.sql.builder.query.aql.SimplePropertyCriterionSubSelect;
import org.artifactory.storage.db.aql.sql.builder.query.sql.SqlTable;
import org.artifactory.storage.db.aql.sql.model.SqlTableEnum;
import org.springframework.test.util.ReflectionTestUtils;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import java.util.List;

import static org.assertj.core.api.Assertions.assertThat;
import static org.springframework.test.util.AssertionErrors.assertTrue;

public class FromLeftOuterJoinToSubSelectTest extends AqlAbstractServiceTest {


    @BeforeClass
    public void setup() {
        importSql("/sql/aql_postgres_property.sql");
        ReflectionTestUtils.setField(aqlService, "permissionProvider", new AdminPermissions());
    }

    @Test
    // ComplexPropertyCriterion
    public void OptimizeByComplexPropertyCriterionNotEquals() {
        AqlQuery aqlQuery = new AqlQuery();
        AqlQueryElement aqlQueryElement =
                new ComplexPropertyCriterion(
                        List.of(AqlDomainEnum.properties),
                        new AqlValue(AqlVariableTypeEnum.string, "start"), new SqlTable(SqlTableEnum.node_props, 100),
                        "$ne", new AqlValue(AqlVariableTypeEnum.string, "df"),
                        new SqlTable(SqlTableEnum.node_props, 100), false);
        aqlQuery.getAqlElements().add(aqlQueryElement);
        new FromLeftOuterJoinToSubSelect().doJob(aqlQuery);
        Object actualValue = aqlQuery.getAqlElements().get(0);
        ComplexPropertyCriterionSubSelect expectedValue = new ComplexPropertyCriterionSubSelect(
                List.of(AqlDomainEnum.properties),
                new AqlValue(AqlVariableTypeEnum.string, "start"), new SqlTable(SqlTableEnum.node_props, 100),
                "$ne", new AqlValue(AqlVariableTypeEnum.string, "df"),
                new SqlTable(SqlTableEnum.node_props, 100), false);
        assertTrue("instance of ComplexPropertyCriterionSubSelect",
                actualValue instanceof ComplexPropertyCriterionSubSelect);
        assertThat(actualValue).usingRecursiveComparison().isEqualTo(expectedValue);
    }

    @Test
    // ComplexPropertyCriterion
    public void OptimizeByComplexPropertyCriterionNotMatch() {
        AqlQuery aqlQuery = new AqlQuery();
        AqlQueryElement aqlQueryElement =
                new ComplexPropertyCriterion(
                        List.of(AqlDomainEnum.properties),
                        new AqlValue(AqlVariableTypeEnum.string, "start"), new SqlTable(SqlTableEnum.node_props, 100),
                        "$nmatch", new AqlValue(AqlVariableTypeEnum.string, "df"),
                        new SqlTable(SqlTableEnum.node_props, 100), false);
        aqlQuery.getAqlElements().add(aqlQueryElement);
        new FromLeftOuterJoinToSubSelect().doJob(aqlQuery);
        Object actualValue = aqlQuery.getAqlElements().get(0);
        ComplexPropertyCriterionSubSelect expectedValue = new ComplexPropertyCriterionSubSelect(
                List.of(AqlDomainEnum.properties),
                new AqlValue(AqlVariableTypeEnum.string, "start"), new SqlTable(SqlTableEnum.node_props, 100),
                "$nmatch", new AqlValue(AqlVariableTypeEnum.string, "df"),
                new SqlTable(SqlTableEnum.node_props, 100), false);
        assertTrue("instance of ComplexPropertyCriterionSubSelect",
                actualValue instanceof ComplexPropertyCriterionSubSelect);
        assertThat(actualValue).usingRecursiveComparison().isEqualTo(expectedValue);
    }


    @Test
    // ComplexPropertyCriterion
    public void OptimizeByComplexPropertyCriterionNotMatchMspOperatorIsTrue() {
        AqlQuery aqlQuery = new AqlQuery();
        AqlQueryElement aqlQueryElement =
                new ComplexPropertyCriterion(
                        List.of(AqlDomainEnum.properties),
                        new AqlValue(AqlVariableTypeEnum.string, "start"), new SqlTable(SqlTableEnum.node_props, 100),
                        "$nmatch", new AqlValue(AqlVariableTypeEnum.string, "df"),
                        new SqlTable(SqlTableEnum.node_props, 100), true);
        aqlQuery.getAqlElements().add(aqlQueryElement);
        new FromLeftOuterJoinToSubSelect().doJob(aqlQuery);
        Object actualValue = aqlQuery.getAqlElements().get(0);
        ComplexPropertyCriterion expectedValue = new ComplexPropertyCriterion(
                List.of(AqlDomainEnum.properties),
                new AqlValue(AqlVariableTypeEnum.string, "start"), new SqlTable(SqlTableEnum.node_props, 100),
                "$nmatch", new AqlValue(AqlVariableTypeEnum.string, "df"),
                new SqlTable(SqlTableEnum.node_props, 100), true);
        assertTrue("instance of ComplexPropertyCriterion",
                actualValue instanceof ComplexPropertyCriterion);
        assertThat(actualValue).usingRecursiveComparison().isEqualTo(expectedValue);
    }

    @Test
    // ComplexPropertyCriterion
    public void OptimizeByComplexPropertyCriterionMatch() {
        AqlQuery aqlQuery = new AqlQuery();
        AqlQueryElement aqlQueryElement =
                new ComplexPropertyCriterion(
                        List.of(AqlDomainEnum.properties),
                        new AqlValue(AqlVariableTypeEnum.string, "start"), new SqlTable(SqlTableEnum.node_props, 100),
                        "$match", new AqlValue(AqlVariableTypeEnum.string, "df"),
                        new SqlTable(SqlTableEnum.node_props, 100), false);
        aqlQuery.getAqlElements().add(aqlQueryElement);
        new FromLeftOuterJoinToSubSelect().doJob(aqlQuery);
        Object actualValue = aqlQuery.getAqlElements().get(0);
        ComplexPropertyCriterion expectedValue = new ComplexPropertyCriterion(
                List.of(AqlDomainEnum.properties),
                new AqlValue(AqlVariableTypeEnum.string, "start"), new SqlTable(SqlTableEnum.node_props, 100),
                "$match", new AqlValue(AqlVariableTypeEnum.string, "df"),
                new SqlTable(SqlTableEnum.node_props, 100), false);
        assertTrue("instance of ComplexPropertyCriterion",
                actualValue instanceof ComplexPropertyCriterion);
        assertThat(actualValue).usingRecursiveComparison().isEqualTo(expectedValue);
    }

    @Test
    // SimplePropertyCriterion
    public void OptimizeBySimplePropertyCriterionNotEquals() {
        AqlQuery aqlQuery = new AqlQuery();
        AqlQueryElement aqlQueryElement =
                new SimplePropertyCriterion(List.of(AqlDomainEnum.properties),
                        new AqlField(AqlPhysicalFieldEnum.propertyKey), new SqlTable(SqlTableEnum.node_props, 100),
                        "$ne",
                        new AqlValue(AqlVariableTypeEnum.string, "*"),
                        new SqlTable(SqlTableEnum.node_props, 100), false);
        ;
        aqlQuery.getAqlElements().add(aqlQueryElement);
        new FromLeftOuterJoinToSubSelect().doJob(aqlQuery);
        Object actualValue = aqlQuery.getAqlElements().get(0);
        SimplePropertyCriterionSubSelect expectedValue =
                new SimplePropertyCriterionSubSelect(List.of(AqlDomainEnum.properties),
                        new AqlField(AqlPhysicalFieldEnum.propertyKey), new SqlTable(SqlTableEnum.node_props, 100),
                        "$ne",
                        new AqlValue(AqlVariableTypeEnum.string, "*"),
                        new SqlTable(SqlTableEnum.node_props, 100), false);
        assertTrue("instance of SimplePropertyCriterionSubSelect",
                actualValue instanceof SimplePropertyCriterionSubSelect);
        assertThat(actualValue).usingRecursiveComparison().isEqualTo(expectedValue);
    }

    @Test
    // SimplePropertyCriterion
    public void OptimizeBySimplePropertyCriterionNotMatch() {
        AqlQuery aqlQuery = new AqlQuery();
        AqlQueryElement aqlQueryElement =
                new SimplePropertyCriterion(List.of(AqlDomainEnum.properties),
                        new AqlField(AqlPhysicalFieldEnum.propertyKey), new SqlTable(SqlTableEnum.node_props, 100),
                        "$nmatch",
                        new AqlValue(AqlVariableTypeEnum.string, "*"),
                        new SqlTable(SqlTableEnum.node_props, 100), false);
        aqlQuery.getAqlElements().add(aqlQueryElement);
        new FromLeftOuterJoinToSubSelect().doJob(aqlQuery);
        Object actualValue = aqlQuery.getAqlElements().get(0);
        SimplePropertyCriterionSubSelect expectedValue =
                new SimplePropertyCriterionSubSelect(List.of(AqlDomainEnum.properties),
                        new AqlField(AqlPhysicalFieldEnum.propertyKey), new SqlTable(SqlTableEnum.node_props, 100),
                        "$nmatch",
                        new AqlValue(AqlVariableTypeEnum.string, "*"),
                        new SqlTable(SqlTableEnum.node_props, 100), false);
        assertTrue("instance of SimplePropertyCriterionSubSelect",
                actualValue instanceof SimplePropertyCriterionSubSelect);
        assertThat(actualValue).usingRecursiveComparison().isEqualTo(expectedValue);
    }

    @Test
    // SimplePropertyCriterion
    public void OptimizeBySimplePropertyCriterionNotMatchMspOperatorIsTrue() {
        AqlQuery aqlQuery = new AqlQuery();
        AqlQueryElement aqlQueryElement =
                new SimplePropertyCriterion(List.of(AqlDomainEnum.properties),
                        new AqlField(AqlPhysicalFieldEnum.propertyKey), new SqlTable(SqlTableEnum.node_props, 100),
                        "$match",
                        new AqlValue(AqlVariableTypeEnum.string, "*"),
                        new SqlTable(SqlTableEnum.node_props, 100), true);
        aqlQuery.getAqlElements().add(aqlQueryElement);
        new FromLeftOuterJoinToSubSelect().doJob(aqlQuery);
        Object actualValue = aqlQuery.getAqlElements().get(0);
        SimplePropertyCriterion expectedValue = new SimplePropertyCriterion(List.of(AqlDomainEnum.properties),
                new AqlField(AqlPhysicalFieldEnum.propertyKey), new SqlTable(SqlTableEnum.node_props, 100),
                "$match",
                new AqlValue(AqlVariableTypeEnum.string, "*"),
                new SqlTable(SqlTableEnum.node_props, 100), true);
        assertTrue("instance of SimplePropertyCriterion",
                actualValue instanceof SimplePropertyCriterion);
        assertThat(actualValue).usingRecursiveComparison().isEqualTo(expectedValue);
    }

    @Test
    // SimplePropertyCriterion
    public void OptimizeBySimplePropertyCriterionMatch() {
        AqlQuery aqlQuery = new AqlQuery();
        AqlQueryElement aqlQueryElement =
                new SimplePropertyCriterion(List.of(AqlDomainEnum.properties),
                        new AqlField(AqlPhysicalFieldEnum.propertyKey), new SqlTable(SqlTableEnum.node_props, 100),
                        "$match",
                        new AqlValue(AqlVariableTypeEnum.string, "*"),
                        new SqlTable(SqlTableEnum.node_props, 100), false);
        aqlQuery.getAqlElements().add(aqlQueryElement);
        new FromLeftOuterJoinToSubSelect().doJob(aqlQuery);
        Object actualValue = aqlQuery.getAqlElements().get(0);
        SimplePropertyCriterion expectedValue = new SimplePropertyCriterion(List.of(AqlDomainEnum.properties),
                new AqlField(AqlPhysicalFieldEnum.propertyKey), new SqlTable(SqlTableEnum.node_props, 100),
                "$match",
                new AqlValue(AqlVariableTypeEnum.string, "*"),
                new SqlTable(SqlTableEnum.node_props, 100), false);
        assertTrue("instance of SimplePropertyCriterion",
                actualValue instanceof SimplePropertyCriterion);
        assertThat(actualValue).usingRecursiveComparison().isEqualTo(expectedValue);
    }
}
