package org.artifactory.ui.rest.resource.admin.configuration.proxies;

import org.artifactory.api.context.ArtifactoryContextThreadBinder;
import org.artifactory.api.security.AuthorizationService;
import org.artifactory.rest.common.service.ArtifactoryRestResponse;
import org.artifactory.rest.exception.AuthorizationRestException;
import org.artifactory.ui.rest.service.admin.configuration.ConfigServiceFactory;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.ValueSource;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;
import org.mockito.Spy;

import javax.ws.rs.core.Response;

import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.anyBoolean;
import static org.mockito.ArgumentMatchers.eq;
import static org.mockito.Mockito.doReturn;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.when;

/**
 * @author Volodymyr Shulga
 */
class ProxyResourceTest {
    @Mock
    protected ConfigServiceFactory configServiceFactory;
    @Mock
    private AuthorizationService authorizationService;
    @Spy
    private ProxyResource proxyResource;
    private AutoCloseable closable;

    @BeforeEach
    public void setUp() throws NoSuchFieldException, IllegalAccessException {
        closable = MockitoAnnotations.openMocks(this);
        proxyResource.setAuthorizationService(authorizationService);
        proxyResource.setConfigServiceFactory(configServiceFactory);
        proxyResource.setArtifactoryResponse(Mockito.mock(ArtifactoryRestResponse.class));
        doReturn(Mockito.mock(Response.class)).when(proxyResource).runService(any(), anyBoolean());
    }

    @AfterEach
    public void tearDown() throws Exception {
        closable.close();
        ArtifactoryContextThreadBinder.unbind();
    }

    @Test
    void getProxiesForNonAuthorizedUser() {
        when(authorizationService.isAnyProjectAdmin()).thenReturn(false);
        Assertions.assertThrows(AuthorizationRestException.class, () -> proxyResource.getProxies());
    }

    @ParameterizedTest
    @ValueSource(booleans = {true, false})
    void ensureIsAdminPassedToRequestAttribute(boolean withPassword) {
        when(authorizationService.isAnyProjectAdmin()).thenReturn(true);
        when(authorizationService.isAdmin()).thenReturn(withPassword);
        proxyResource.getProxies();
        Mockito.verify(proxyResource, times(1)).runService(any(), eq(withPassword));
    }
}